@@ -32,5 +32,9 @@ public interface PolarisCatalog {
   boolean dropGenericTable(TableIdentifier identifier);
 
   GenericTable createGenericTable(
-      TableIdentifier identifier, String format, String doc, Map<String, String> props);
+      TableIdentifier identifier,
+      String format,
+      String baseLocation,
+      String doc,
+      Map<String, String> props);
 }
