@@ -19,7 +19,7 @@
 
 plugins {
   id("polaris-client")
-  alias(libs.plugins.jandex)
+  id("org.kordamp.gradle.jandex")
 }
 
 dependencies {
@@ -36,18 +36,21 @@ dependencies {
   implementation("com.fasterxml.jackson.core:jackson-annotations")
   implementation("com.fasterxml.jackson.core:jackson-core")
   implementation("com.fasterxml.jackson.core:jackson-databind")
+  implementation("com.fasterxml.jackson.dataformat:jackson-dataformat-smile")
+  runtimeOnly("com.fasterxml.jackson.datatype:jackson-datatype-guava")
+  runtimeOnly("com.fasterxml.jackson.datatype:jackson-datatype-jdk8")
+  runtimeOnly("com.fasterxml.jackson.datatype:jackson-datatype-jsr310")
+
   implementation(libs.caffeine)
-  implementation(libs.commons.lang3)
-  implementation(libs.commons.codec1)
+  implementation(libs.commons.codec)
   implementation(libs.guava)
   implementation(libs.slf4j.api)
-  compileOnly(libs.spotbugs.annotations)
 
   compileOnly(project(":polaris-immutables"))
   annotationProcessor(project(":polaris-immutables", configuration = "processor"))
 
   constraints {
-    implementation("org.xerial.snappy:snappy-java:1.1.10.7") {
+    implementation("org.xerial.snappy:snappy-java:1.1.10.8") {
       because("Vulnerability detected in 1.1.8.2")
     }
     implementation("org.codehaus.jettison:jettison:1.5.4") {
@@ -56,10 +59,10 @@ dependencies {
     implementation("org.apache.commons:commons-configuration2:2.12.0") {
       because("Vulnerability detected in 2.8.0")
     }
-    implementation("org.apache.commons:commons-compress:1.27.1") {
+    implementation("org.apache.commons:commons-compress:1.28.0") {
       because("Vulnerability detected in 1.21")
     }
-    implementation("com.nimbusds:nimbus-jose-jwt:10.3") {
+    implementation("com.nimbusds:nimbus-jose-jwt:10.4.2") {
       because("Vulnerability detected in 9.8.1")
     }
   }
@@ -84,10 +87,10 @@ dependencies {
   implementation("com.azure:azure-identity")
   implementation("com.azure:azure-storage-file-datalake")
   constraints {
-    implementation("io.netty:netty-codec-http2:4.2.2.Final") {
+    implementation("io.netty:netty-codec-http2:4.2.4.Final") {
       because("Vulnerability detected in 4.1.72")
     }
-    implementation("io.projectreactor.netty:reactor-netty-http:1.2.7") {
+    implementation("io.projectreactor.netty:reactor-netty-http:1.2.9") {
       because("Vulnerability detected in 1.0.45")
     }
   }
@@ -96,6 +99,9 @@ dependencies {
   implementation(platform(libs.google.cloud.storage.bom))
   implementation("com.google.cloud:google-cloud-storage")
 
+  testCompileOnly(project(":polaris-immutables"))
+  testAnnotationProcessor(project(":polaris-immutables", configuration = "processor"))
+
   testFixturesApi("com.fasterxml.jackson.core:jackson-core")
   testFixturesApi("com.fasterxml.jackson.core:jackson-databind")
   testFixturesApi(libs.commons.lang3)
