@@ -20,6 +20,7 @@ package org.apache.polaris.core.persistence.resolver;
 
 import com.google.common.collect.HashMultimap;
 import com.google.common.collect.Multimap;
+import jakarta.annotation.Nullable;
 import jakarta.ws.rs.core.SecurityContext;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -28,13 +29,12 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import org.apache.polaris.core.PolarisDiagnostics;
-import org.apache.polaris.core.auth.AuthenticatedPolarisPrincipal;
+import org.apache.polaris.core.auth.PolarisPrincipal;
 import org.apache.polaris.core.context.CallContext;
 import org.apache.polaris.core.entity.PolarisBaseEntity;
 import org.apache.polaris.core.entity.PolarisEntityConstants;
 import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
-import org.apache.polaris.core.persistence.PolarisEntityManager;
 import org.apache.polaris.core.persistence.PolarisResolvedPathWrapper;
 import org.apache.polaris.core.persistence.ResolvedPolarisEntity;
 import org.slf4j.Logger;
@@ -51,7 +51,7 @@ import org.slf4j.LoggerFactory;
 public class PolarisResolutionManifest implements PolarisResolutionManifestCatalogView {
   private static final Logger LOGGER = LoggerFactory.getLogger(PolarisResolutionManifest.class);
 
-  private final PolarisEntityManager entityManager;
+  private final ResolverFactory resolverFactory;
   private final CallContext callContext;
   private final SecurityContext securityContext;
   private final String catalogName;
@@ -64,13 +64,6 @@ public class PolarisResolutionManifest implements PolarisResolutionManifestCatal
 
   private final Map<Object, ResolverPath> passthroughPaths = new HashMap<>();
 
-  // For applicable operations, this represents the topmost root entity which services as an
-  // authorization parent for all other entities that reside at the root level, such as
-  // Catalog, Principal, and PrincipalRole.
-  // This simulated entity will be used if the actual resolver fails to resolve the rootContainer
-  // on the backend due to compatibility mismatches.
-  private ResolvedPolarisEntity simulatedResolvedRootContainerEntity = null;
-
   private int currentPathIndex = 0;
 
   // Set when resolveAll is called
@@ -78,18 +71,19 @@ public class PolarisResolutionManifest implements PolarisResolutionManifestCatal
 
   public PolarisResolutionManifest(
       CallContext callContext,
-      PolarisEntityManager entityManager,
+      ResolverFactory resolverFactory,
       SecurityContext securityContext,
       String catalogName) {
-    this.entityManager = entityManager;
     this.callContext = callContext;
+    this.resolverFactory = resolverFactory;
     this.catalogName = catalogName;
-    this.primaryResolver = entityManager.prepareResolver(callContext, securityContext, catalogName);
+    this.primaryResolver =
+        resolverFactory.createResolver(callContext, securityContext, catalogName);
     this.diagnostics = callContext.getPolarisCallContext().getDiagServices();
     this.diagnostics.checkNotNull(securityContext, "null_security_context_for_resolution_manifest");
     this.securityContext = securityContext;
     diagnostics.check(
-        securityContext.getUserPrincipal() instanceof AuthenticatedPolarisPrincipal,
+        securityContext.getUserPrincipal() instanceof PolarisPrincipal,
         "invalid_principal_type_for_resolution_manifest",
         "principal={}",
         securityContext.getUserPrincipal());
@@ -193,7 +187,7 @@ public class PolarisResolutionManifest implements PolarisResolutionManifestCatal
 
     // Run a single-use Resolver for this path.
     Resolver passthroughResolver =
-        entityManager.prepareResolver(callContext, securityContext, catalogName);
+        resolverFactory.createResolver(callContext, securityContext, catalogName);
     passthroughResolver.addPath(requestedPath);
     ResolverStatus status = passthroughResolver.resolveAll();
 
@@ -268,12 +262,7 @@ public class PolarisResolutionManifest implements PolarisResolutionManifestCatal
     return activatedEntities;
   }
 
-  public void setSimulatedResolvedRootContainerEntity(
-      ResolvedPolarisEntity simulatedResolvedRootContainerEntity) {
-    this.simulatedResolvedRootContainerEntity = simulatedResolvedRootContainerEntity;
-  }
-
-  private ResolvedPolarisEntity getResolvedRootContainerEntity() {
+  private @Nullable ResolvedPolarisEntity getResolvedRootContainerEntity() {
     if (primaryResolverStatus.getStatus() != ResolverStatus.StatusEnum.SUCCESS) {
       return null;
     }
@@ -281,8 +270,10 @@ public class PolarisResolutionManifest implements PolarisResolutionManifestCatal
         primaryResolver.getResolvedEntity(
             PolarisEntityType.ROOT, PolarisEntityConstants.getRootContainerName());
     if (resolvedEntity == null) {
-      LOGGER.debug("Failed to find rootContainer, so using simulated rootContainer instead.");
-      return simulatedResolvedRootContainerEntity;
+      LOGGER.warn(
+          "Failed to find rootContainer for realm: {} and catalog: {}",
+          callContext.getRealmContext().getRealmIdentifier(),
+          catalogName);
     }
     return resolvedEntity;
   }
