@@ -20,14 +20,13 @@ package org.apache.polaris.core.storage.cache;
 
 import static org.apache.polaris.core.persistence.PrincipalSecretsGenerator.RANDOM_SECRETS;
 
-import com.google.common.collect.ImmutableMap;
 import jakarta.annotation.Nonnull;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.EnumMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
+import java.util.Set;
 import org.apache.iceberg.exceptions.UnprocessableEntityException;
 import org.apache.polaris.core.PolarisCallContext;
 import org.apache.polaris.core.PolarisDefaultDiagServiceImpl;
@@ -38,7 +37,6 @@ import org.apache.polaris.core.entity.PolarisEntityConstants;
 import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
 import org.apache.polaris.core.persistence.PolarisMetaStoreManager;
-import org.apache.polaris.core.persistence.PolarisObjectMapperUtil;
 import org.apache.polaris.core.persistence.dao.entity.BaseResult;
 import org.apache.polaris.core.persistence.dao.entity.ScopedCredentialsResult;
 import org.apache.polaris.core.persistence.transactional.TransactionalPersistence;
@@ -46,7 +44,6 @@ import org.apache.polaris.core.persistence.transactional.TreeMapMetaStore;
 import org.apache.polaris.core.persistence.transactional.TreeMapTransactionalPersistenceImpl;
 import org.apache.polaris.core.storage.AccessConfig;
 import org.apache.polaris.core.storage.StorageAccessProperty;
-import org.apache.polaris.core.storage.azure.AzureLocation;
 import org.assertj.core.api.Assertions;
 import org.junit.jupiter.api.RepeatedTest;
 import org.junit.jupiter.api.Test;
@@ -54,13 +51,11 @@ import org.mockito.Mockito;
 
 public class StorageCredentialCacheTest {
 
-  // polaris call context
   private final PolarisCallContext callCtx;
-
-  // the meta store manager
+  private final StorageCredentialCacheConfig storageCredentialCacheConfig;
   private final PolarisMetaStoreManager metaStoreManager;
 
-  StorageCredentialCache storageCredentialCache;
+  private StorageCredentialCache storageCredentialCache;
 
   public StorageCredentialCacheTest() {
     // diag services
@@ -69,15 +64,21 @@ public class StorageCredentialCacheTest {
     TreeMapMetaStore store = new TreeMapMetaStore(diagServices);
     // to interact with the metastore
     TransactionalPersistence metaStore =
-        new TreeMapTransactionalPersistenceImpl(store, Mockito.mock(), RANDOM_SECRETS);
+        new TreeMapTransactionalPersistenceImpl(
+            diagServices, store, Mockito.mock(), RANDOM_SECRETS);
     callCtx = new PolarisCallContext(() -> "testRealm", metaStore, diagServices);
+    storageCredentialCacheConfig = () -> 10_000;
     metaStoreManager = Mockito.mock(PolarisMetaStoreManager.class);
-    storageCredentialCache = new StorageCredentialCache();
+    storageCredentialCache = newStorageCredentialCache();
+  }
+
+  private StorageCredentialCache newStorageCredentialCache() {
+    return new StorageCredentialCache(storageCredentialCacheConfig);
   }
 
   @Test
   public void testBadResult() {
-    storageCredentialCache = new StorageCredentialCache();
+    storageCredentialCache = newStorageCredentialCache();
     ScopedCredentialsResult badResult =
         new ScopedCredentialsResult(
             BaseResult.ReturnStatus.SUBSCOPE_CREDS_ERROR, "extra_error_info");
@@ -89,7 +90,8 @@ public class StorageCredentialCacheTest {
                 Mockito.any(),
                 Mockito.anyBoolean(),
                 Mockito.anySet(),
-                Mockito.anySet()))
+                Mockito.anySet(),
+                Mockito.any()))
         .thenReturn(badResult);
     PolarisEntity polarisEntity =
         new PolarisEntity(
@@ -102,15 +104,16 @@ public class StorageCredentialCacheTest {
                     callCtx,
                     polarisEntity,
                     true,
-                    new HashSet<>(Arrays.asList("s3://bucket1/path")),
-                    new HashSet<>(Arrays.asList("s3://bucket3/path"))))
+                    Set.of("s3://bucket1/path"),
+                    Set.of("s3://bucket3/path"),
+                    Optional.empty()))
         .isInstanceOf(UnprocessableEntityException.class)
         .hasMessage("Failed to get subscoped credentials: extra_error_info");
   }
 
   @Test
   public void testCacheHit() {
-    storageCredentialCache = new StorageCredentialCache();
+    storageCredentialCache = newStorageCredentialCache();
     List<ScopedCredentialsResult> mockedScopedCreds =
         getFakeScopedCreds(3, /* expireSoon= */ false);
     Mockito.when(
@@ -121,7 +124,8 @@ public class StorageCredentialCacheTest {
                 Mockito.any(),
                 Mockito.anyBoolean(),
                 Mockito.anySet(),
-                Mockito.anySet()))
+                Mockito.anySet(),
+                Mockito.any()))
         .thenReturn(mockedScopedCreds.get(0))
         .thenReturn(mockedScopedCreds.get(1))
         .thenReturn(mockedScopedCreds.get(1));
@@ -136,8 +140,9 @@ public class StorageCredentialCacheTest {
         callCtx,
         polarisEntity,
         true,
-        new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-        new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")));
+        Set.of("s3://bucket1/path", "s3://bucket2/path"),
+        Set.of("s3://bucket3/path", "s3://bucket4/path"),
+        Optional.empty());
     Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(1);
 
     // subscope for the same entity and same allowed locations, will hit the cache
@@ -146,14 +151,15 @@ public class StorageCredentialCacheTest {
         callCtx,
         polarisEntity,
         true,
-        new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-        new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")));
+        Set.of("s3://bucket1/path", "s3://bucket2/path"),
+        Set.of("s3://bucket3/path", "s3://bucket4/path"),
+        Optional.empty());
     Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(1);
   }
 
   @RepeatedTest(10)
   public void testCacheEvict() throws InterruptedException {
-    storageCredentialCache = new StorageCredentialCache();
+    storageCredentialCache = newStorageCredentialCache();
     List<ScopedCredentialsResult> mockedScopedCreds = getFakeScopedCreds(3, /* expireSoon= */ true);
 
     Mockito.when(
@@ -164,7 +170,8 @@ public class StorageCredentialCacheTest {
                 Mockito.any(),
                 Mockito.anyBoolean(),
                 Mockito.anySet(),
-                Mockito.anySet()))
+                Mockito.anySet(),
+                Mockito.any()))
         .thenReturn(mockedScopedCreds.get(0))
         .thenReturn(mockedScopedCreds.get(1))
         .thenReturn(mockedScopedCreds.get(2));
@@ -173,12 +180,13 @@ public class StorageCredentialCacheTest {
             1, 2, PolarisEntityType.CATALOG, PolarisEntitySubType.ICEBERG_TABLE, 0, "name");
     PolarisEntity polarisEntity = new PolarisEntity(baseEntity);
     StorageCredentialCacheKey cacheKey =
-        new StorageCredentialCacheKey(
+        StorageCredentialCacheKey.of(
+            callCtx.getRealmContext().getRealmIdentifier(),
             polarisEntity,
             true,
-            new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-            new HashSet<>(Arrays.asList("s3://bucket/path")),
-            callCtx);
+            Set.of("s3://bucket1/path", "s3://bucket2/path"),
+            Set.of("s3://bucket/path"),
+            Optional.empty());
 
     // the entry will be evicted immediately because the token is expired
     storageCredentialCache.getOrGenerateSubScopeCreds(
@@ -186,8 +194,9 @@ public class StorageCredentialCacheTest {
         callCtx,
         polarisEntity,
         true,
-        new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-        new HashSet<>(Arrays.asList("s3://bucket/path")));
+        Set.of("s3://bucket1/path", "s3://bucket2/path"),
+        Set.of("s3://bucket/path"),
+        Optional.empty());
     Assertions.assertThat(storageCredentialCache.getIfPresent(cacheKey)).isNull();
 
     storageCredentialCache.getOrGenerateSubScopeCreds(
@@ -195,8 +204,9 @@ public class StorageCredentialCacheTest {
         callCtx,
         polarisEntity,
         true,
-        new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-        new HashSet<>(Arrays.asList("s3://bucket/path")));
+        Set.of("s3://bucket1/path", "s3://bucket2/path"),
+        Set.of("s3://bucket/path"),
+        Optional.empty());
     Assertions.assertThat(storageCredentialCache.getIfPresent(cacheKey)).isNull();
 
     storageCredentialCache.getOrGenerateSubScopeCreds(
@@ -204,14 +214,15 @@ public class StorageCredentialCacheTest {
         callCtx,
         polarisEntity,
         true,
-        new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-        new HashSet<>(Arrays.asList("s3://bucket/path")));
+        Set.of("s3://bucket1/path", "s3://bucket2/path"),
+        Set.of("s3://bucket/path"),
+        Optional.empty());
     Assertions.assertThat(storageCredentialCache.getIfPresent(cacheKey)).isNull();
   }
 
   @Test
   public void testCacheGenerateNewEntries() {
-    storageCredentialCache = new StorageCredentialCache();
+    storageCredentialCache = newStorageCredentialCache();
     List<ScopedCredentialsResult> mockedScopedCreds =
         getFakeScopedCreds(3, /* expireSoon= */ false);
     Mockito.when(
@@ -222,7 +233,8 @@ public class StorageCredentialCacheTest {
                 Mockito.any(),
                 Mockito.anyBoolean(),
                 Mockito.anySet(),
-                Mockito.anySet()))
+                Mockito.anySet(),
+                Mockito.any()))
         .thenReturn(mockedScopedCreds.get(0))
         .thenReturn(mockedScopedCreds.get(1))
         .thenReturn(mockedScopedCreds.get(2));
@@ -235,8 +247,9 @@ public class StorageCredentialCacheTest {
           callCtx,
           entity,
           true,
-          new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://bucket/path")));
+          Set.of("s3://bucket1/path", "s3://bucket2/path"),
+          Set.of("s3://bucket/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(++cacheSize);
     }
     // update the entity's storage config, since StorageConfig changed, cache will generate new
@@ -245,15 +258,16 @@ public class StorageCredentialCacheTest {
       Map<String, String> internalMap = entity.getPropertiesAsMap();
       internalMap.put(
           PolarisEntityConstants.getStorageConfigInfoPropertyName(), "newStorageConfig");
-      entity.setInternalProperties(
-          PolarisObjectMapperUtil.serializeProperties(callCtx, internalMap));
+      PolarisBaseEntity updateEntity =
+          new PolarisBaseEntity.Builder(entity).internalPropertiesAsMap(internalMap).build();
       storageCredentialCache.getOrGenerateSubScopeCreds(
           metaStoreManager,
           callCtx,
-          entity,
+          PolarisEntity.of(updateEntity),
           /* allowedListAction= */ true,
-          new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://bucket/path")));
+          Set.of("s3://bucket1/path", "s3://bucket2/path"),
+          Set.of("s3://bucket/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(++cacheSize);
     }
     // allowedListAction changed to different value FALSE, will generate new entry
@@ -263,8 +277,9 @@ public class StorageCredentialCacheTest {
           callCtx,
           entity,
           /* allowedListAction= */ false,
-          new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://bucket/path")));
+          Set.of("s3://bucket1/path", "s3://bucket2/path"),
+          Set.of("s3://bucket/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(++cacheSize);
     }
     // different allowedWriteLocations, will generate new entry
@@ -274,8 +289,9 @@ public class StorageCredentialCacheTest {
           callCtx,
           entity,
           /* allowedListAction= */ false,
-          new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://differentbucket/path")));
+          Set.of("s3://bucket1/path", "s3://bucket2/path"),
+          Set.of("s3://differentbucket/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(++cacheSize);
     }
     // different allowedReadLocations, will generate new try
@@ -283,22 +299,23 @@ public class StorageCredentialCacheTest {
       Map<String, String> internalMap = entity.getPropertiesAsMap();
       internalMap.put(
           PolarisEntityConstants.getStorageConfigInfoPropertyName(), "newStorageConfig");
-      entity.setInternalProperties(
-          PolarisObjectMapperUtil.serializeProperties(callCtx, internalMap));
+      PolarisBaseEntity updateEntity =
+          new PolarisBaseEntity.Builder(entity).internalPropertiesAsMap(internalMap).build();
       storageCredentialCache.getOrGenerateSubScopeCreds(
           metaStoreManager,
           callCtx,
-          entity,
+          PolarisEntity.of(updateEntity),
           /* allowedListAction= */ false,
-          new HashSet<>(Arrays.asList("s3://differentbucket/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://bucket/path")));
+          Set.of("s3://differentbucket/path", "s3://bucket2/path"),
+          Set.of("s3://bucket/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(++cacheSize);
     }
   }
 
   @Test
   public void testCacheNotAffectedBy() {
-    storageCredentialCache = new StorageCredentialCache();
+    storageCredentialCache = newStorageCredentialCache();
     List<ScopedCredentialsResult> mockedScopedCreds =
         getFakeScopedCreds(3, /* expireSoon= */ false);
 
@@ -310,7 +327,8 @@ public class StorageCredentialCacheTest {
                 Mockito.any(),
                 Mockito.anyBoolean(),
                 Mockito.anySet(),
-                Mockito.anySet()))
+                Mockito.anySet(),
+                Mockito.any()))
         .thenReturn(mockedScopedCreds.get(0))
         .thenReturn(mockedScopedCreds.get(1))
         .thenReturn(mockedScopedCreds.get(2));
@@ -321,59 +339,60 @@ public class StorageCredentialCacheTest {
           callCtx,
           entity,
           true,
-          new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")));
+          Set.of("s3://bucket1/path", "s3://bucket2/path"),
+          Set.of("s3://bucket3/path", "s3://bucket4/path"),
+          Optional.empty());
     }
     Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(entityList.size());
 
     // entity ID does not affect the cache
     for (PolarisEntity entity : entityList) {
-      entity.setId(1234);
       storageCredentialCache.getOrGenerateSubScopeCreds(
           metaStoreManager,
           callCtx,
-          entity,
+          new PolarisEntity(new PolarisBaseEntity.Builder(entity).id(1234).build()),
           true,
-          new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")));
+          Set.of("s3://bucket1/path", "s3://bucket2/path"),
+          Set.of("s3://bucket3/path", "s3://bucket4/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(entityList.size());
     }
 
     // other property changes does not affect the cache
     for (PolarisEntity entity : entityList) {
-      entity.setEntityVersion(5);
       storageCredentialCache.getOrGenerateSubScopeCreds(
           metaStoreManager,
           callCtx,
-          entity,
+          new PolarisEntity(new PolarisBaseEntity.Builder(entity).entityVersion(5).build()),
           true,
-          new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-          new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")));
+          Set.of("s3://bucket1/path", "s3://bucket2/path"),
+          Set.of("s3://bucket3/path", "s3://bucket4/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(entityList.size());
     }
     // order of the allowedReadLocations does not affect the cache
     for (PolarisEntity entity : entityList) {
-      entity.setEntityVersion(5);
       storageCredentialCache.getOrGenerateSubScopeCreds(
           metaStoreManager,
           callCtx,
-          entity,
+          new PolarisEntity(new PolarisBaseEntity.Builder(entity).entityVersion(5).build()),
           true,
-          new HashSet<>(Arrays.asList("s3://bucket2/path", "s3://bucket1/path")),
-          new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")));
+          Set.of("s3://bucket2/path", "s3://bucket1/path"),
+          Set.of("s3://bucket3/path", "s3://bucket4/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(entityList.size());
     }
 
     // order of the allowedWriteLocations does not affect the cache
     for (PolarisEntity entity : entityList) {
-      entity.setEntityVersion(5);
       storageCredentialCache.getOrGenerateSubScopeCreds(
           metaStoreManager,
           callCtx,
-          entity,
+          new PolarisEntity(new PolarisBaseEntity.Builder(entity).entityVersion(5).build()),
           true,
-          new HashSet<>(Arrays.asList("s3://bucket2/path", "s3://bucket1/path")),
-          new HashSet<>(Arrays.asList("s3://bucket4/path", "s3://bucket3/path")));
+          Set.of("s3://bucket2/path", "s3://bucket1/path"),
+          Set.of("s3://bucket4/path", "s3://bucket3/path"),
+          Optional.empty());
       Assertions.assertThat(storageCredentialCache.getEstimatedSize()).isEqualTo(entityList.size());
     }
   }
@@ -395,30 +414,26 @@ public class StorageCredentialCacheTest {
               : String.valueOf(Long.MAX_VALUE);
       res.add(
           new ScopedCredentialsResult(
-              new EnumMap<>(
-                  ImmutableMap.<StorageAccessProperty, String>builder()
-                      .put(StorageAccessProperty.AWS_KEY_ID, "key_id_" + finalI)
-                      .put(StorageAccessProperty.AWS_SECRET_KEY, "key_secret_" + finalI)
-                      .put(StorageAccessProperty.AWS_SESSION_TOKEN_EXPIRES_AT_MS, expireTime)
-                      .put(StorageAccessProperty.EXPIRATION_TIME, expireTime)
-                      .buildOrThrow())));
+              AccessConfig.builder()
+                  .put(StorageAccessProperty.AWS_KEY_ID, "key_id_" + finalI)
+                  .put(StorageAccessProperty.AWS_SECRET_KEY, "key_secret_" + finalI)
+                  .put(StorageAccessProperty.AWS_SESSION_TOKEN_EXPIRES_AT_MS, expireTime)
+                  .put(StorageAccessProperty.EXPIRATION_TIME, expireTime)
+                  .build()));
       if (res.size() == number) return res;
       res.add(
           new ScopedCredentialsResult(
-              new EnumMap<>(
-                  ImmutableMap.<StorageAccessProperty, String>builder()
-                      .put(StorageAccessProperty.AZURE_SAS_TOKEN, "sas_token_" + finalI)
-                      .put(StorageAccessProperty.AZURE_ACCOUNT_HOST, "account_host")
-                      .put(StorageAccessProperty.EXPIRATION_TIME, expireTime)
-                      .buildOrThrow())));
+              AccessConfig.builder()
+                  .put(StorageAccessProperty.AZURE_SAS_TOKEN, "sas_token_" + finalI)
+                  .put(StorageAccessProperty.EXPIRATION_TIME, expireTime)
+                  .build()));
       if (res.size() == number) return res;
       res.add(
           new ScopedCredentialsResult(
-              new EnumMap<>(
-                  ImmutableMap.<StorageAccessProperty, String>builder()
-                      .put(StorageAccessProperty.GCS_ACCESS_TOKEN, "gcs_token_" + finalI)
-                      .put(StorageAccessProperty.GCS_ACCESS_TOKEN_EXPIRES_AT, expireTime)
-                      .buildOrThrow())));
+              AccessConfig.builder()
+                  .put(StorageAccessProperty.GCS_ACCESS_TOKEN, "gcs_token_" + finalI)
+                  .put(StorageAccessProperty.GCS_ACCESS_TOKEN_EXPIRES_AT, expireTime)
+                  .build()));
     }
     return res;
   }
@@ -441,106 +456,16 @@ public class StorageCredentialCacheTest {
     return Arrays.asList(polarisEntity1, polarisEntity2, polarisEntity3);
   }
 
-  @Test
-  public void testAzureCredentialFormatting() {
-    storageCredentialCache = new StorageCredentialCache();
-    List<ScopedCredentialsResult> mockedScopedCreds =
-        List.of(
-            new ScopedCredentialsResult(
-                new EnumMap<>(
-                    ImmutableMap.<StorageAccessProperty, String>builder()
-                        .put(StorageAccessProperty.AZURE_SAS_TOKEN, "sas_token_azure_1")
-                        .put(StorageAccessProperty.AZURE_ACCOUNT_HOST, "some_account")
-                        .put(StorageAccessProperty.EXPIRATION_TIME, String.valueOf(Long.MAX_VALUE))
-                        .buildOrThrow())),
-            new ScopedCredentialsResult(
-                new EnumMap<>(
-                    ImmutableMap.<StorageAccessProperty, String>builder()
-                        .put(StorageAccessProperty.AZURE_SAS_TOKEN, "sas_token_azure_2")
-                        .put(
-                            StorageAccessProperty.AZURE_ACCOUNT_HOST,
-                            "some_account." + AzureLocation.ADLS_ENDPOINT)
-                        .put(StorageAccessProperty.EXPIRATION_TIME, String.valueOf(Long.MAX_VALUE))
-                        .buildOrThrow())),
-            new ScopedCredentialsResult(
-                new EnumMap<>(
-                    ImmutableMap.<StorageAccessProperty, String>builder()
-                        .put(StorageAccessProperty.AZURE_SAS_TOKEN, "sas_token_azure_3")
-                        .put(
-                            StorageAccessProperty.AZURE_ACCOUNT_HOST,
-                            "some_account." + AzureLocation.BLOB_ENDPOINT)
-                        .put(StorageAccessProperty.EXPIRATION_TIME, String.valueOf(Long.MAX_VALUE))
-                        .buildOrThrow())));
-
-    Mockito.when(
-            metaStoreManager.getSubscopedCredsForEntity(
-                Mockito.any(),
-                Mockito.anyLong(),
-                Mockito.anyLong(),
-                Mockito.any(),
-                Mockito.anyBoolean(),
-                Mockito.anySet(),
-                Mockito.anySet()))
-        .thenReturn(mockedScopedCreds.get(0))
-        .thenReturn(mockedScopedCreds.get(1))
-        .thenReturn(mockedScopedCreds.get(2));
-    List<PolarisEntity> entityList = getPolarisEntities();
-
-    Map<String, String> noSuffixResult =
-        storageCredentialCache
-            .getOrGenerateSubScopeCreds(
-                metaStoreManager,
-                callCtx,
-                entityList.get(0),
-                true,
-                new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-                new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")))
-            .credentials();
-    Assertions.assertThat(noSuffixResult.size()).isEqualTo(2);
-    Assertions.assertThat(noSuffixResult).containsKey("adls.sas-token.some_account");
-
-    Map<String, String> adlsSuffixResult =
-        storageCredentialCache
-            .getOrGenerateSubScopeCreds(
-                metaStoreManager,
-                callCtx,
-                entityList.get(1),
-                true,
-                new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-                new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")))
-            .credentials();
-    Assertions.assertThat(adlsSuffixResult.size()).isEqualTo(3);
-    Assertions.assertThat(adlsSuffixResult).containsKey("adls.sas-token.some_account");
-    Assertions.assertThat(adlsSuffixResult)
-        .containsKey("adls.sas-token.some_account." + AzureLocation.ADLS_ENDPOINT);
-
-    Map<String, String> blobSuffixResult =
-        storageCredentialCache
-            .getOrGenerateSubScopeCreds(
-                metaStoreManager,
-                callCtx,
-                entityList.get(2),
-                true,
-                new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-                new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")))
-            .credentials();
-    Assertions.assertThat(blobSuffixResult.size()).isEqualTo(3);
-    Assertions.assertThat(blobSuffixResult).containsKey("adls.sas-token.some_account");
-    Assertions.assertThat(blobSuffixResult)
-        .containsKey("adls.sas-token.some_account." + AzureLocation.BLOB_ENDPOINT);
-  }
-
   @Test
   public void testExtraProperties() {
-    storageCredentialCache = new StorageCredentialCache();
+    storageCredentialCache = newStorageCredentialCache();
     ScopedCredentialsResult properties =
         new ScopedCredentialsResult(
-            new EnumMap<>(
-                ImmutableMap.<StorageAccessProperty, String>builder()
-                    .put(StorageAccessProperty.AWS_SECRET_KEY, "super-secret-123")
-                    .put(StorageAccessProperty.AWS_ENDPOINT, "test-endpoint1")
-                    .put(StorageAccessProperty.AWS_PATH_STYLE_ACCESS, "true")
-                    .buildOrThrow()));
+            AccessConfig.builder()
+                .put(StorageAccessProperty.AWS_SECRET_KEY, "super-secret-123")
+                .put(StorageAccessProperty.AWS_ENDPOINT, "test-endpoint1")
+                .put(StorageAccessProperty.AWS_PATH_STYLE_ACCESS, "true")
+                .build());
     Mockito.when(
             metaStoreManager.getSubscopedCredsForEntity(
                 Mockito.any(),
@@ -549,7 +474,8 @@ public class StorageCredentialCacheTest {
                 Mockito.any(),
                 Mockito.anyBoolean(),
                 Mockito.anySet(),
-                Mockito.anySet()))
+                Mockito.anySet(),
+                Mockito.any()))
         .thenReturn(properties);
     List<PolarisEntity> entityList = getPolarisEntities();
 
@@ -559,8 +485,9 @@ public class StorageCredentialCacheTest {
             callCtx,
             entityList.get(0),
             true,
-            new HashSet<>(Arrays.asList("s3://bucket1/path", "s3://bucket2/path")),
-            new HashSet<>(Arrays.asList("s3://bucket3/path", "s3://bucket4/path")));
+            Set.of("s3://bucket1/path", "s3://bucket2/path"),
+            Set.of("s3://bucket3/path", "s3://bucket4/path"),
+            Optional.empty());
     Assertions.assertThat(config.credentials())
         .containsExactly(Map.entry("s3.secret-access-key", "super-secret-123"));
     Assertions.assertThat(config.extraProperties())
