@@ -35,13 +35,12 @@ import java.util.function.Function;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 import org.apache.polaris.core.PolarisCallContext;
-import org.apache.polaris.core.context.CallContext;
 import org.apache.polaris.core.entity.AsyncTaskType;
-import org.apache.polaris.core.entity.EntityNameLookupRecord;
 import org.apache.polaris.core.entity.PolarisBaseEntity;
 import org.apache.polaris.core.entity.PolarisEntity;
 import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
+import org.apache.polaris.core.entity.PolarisTaskConstants;
 import org.apache.polaris.core.entity.TaskEntity;
 import org.apache.polaris.core.persistence.pagination.PageToken;
 import org.assertj.core.api.Assertions;
@@ -64,7 +63,7 @@ import org.threeten.extra.MutableClock;
  */
 public abstract class BasePolarisMetaStoreManagerTest {
 
-  protected final MutableClock timeSource = MutableClock.of(Instant.now(), ZoneOffset.UTC);
+  protected final MutableClock clock = MutableClock.of(Instant.now(), ZoneOffset.UTC);
 
   private PolarisTestMetaStoreManager polarisTestMetaStoreManager;
 
@@ -104,9 +103,6 @@ public abstract class BasePolarisMetaStoreManagerTest {
   @Test
   protected void testCreateEntities() {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
-    if (CallContext.getCurrentContext() == null) {
-      CallContext.setCurrentContext(polarisTestMetaStoreManager.polarisCallContext);
-    }
     TaskEntity task1 = createTask("task1", 100L);
     TaskEntity task2 = createTask("task2", 101L);
     List<PolarisBaseEntity> createdEntities =
@@ -121,41 +117,22 @@ public abstract class BasePolarisMetaStoreManagerTest {
         .extracting(PolarisEntity::toCore)
         .containsExactly(PolarisEntity.toCore(task1), PolarisEntity.toCore(task2));
 
-    List<EntityNameLookupRecord> listedEntities =
-        metaStoreManager
-            .listEntities(
-                polarisTestMetaStoreManager.polarisCallContext,
-                null,
-                PolarisEntityType.TASK,
-                PolarisEntitySubType.NULL_SUBTYPE,
-                PageToken.readEverything())
-            .getEntities();
+    List<PolarisBaseEntity> listedEntities =
+        metaStoreManager.loadEntitiesAll(
+            polarisTestMetaStoreManager.polarisCallContext,
+            null,
+            PolarisEntityType.TASK,
+            PolarisEntitySubType.NULL_SUBTYPE);
     Assertions.assertThat(listedEntities)
         .isNotNull()
         .hasSize(2)
-        .containsExactly(
-            new EntityNameLookupRecord(
-                task1.getCatalogId(),
-                task1.getId(),
-                task1.getParentId(),
-                task1.getName(),
-                task1.getTypeCode(),
-                task1.getSubTypeCode()),
-            new EntityNameLookupRecord(
-                task2.getCatalogId(),
-                task2.getId(),
-                task2.getParentId(),
-                task2.getName(),
-                task2.getTypeCode(),
-                task2.getSubTypeCode()));
+        .extracting(PolarisEntity::toCore)
+        .containsExactly(PolarisEntity.toCore(task1), PolarisEntity.toCore(task2));
   }
 
   @Test
   protected void testCreateEntitiesAlreadyExisting() {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
-    if (CallContext.getCurrentContext() == null) {
-      CallContext.setCurrentContext(polarisTestMetaStoreManager.polarisCallContext);
-    }
     TaskEntity task1 = createTask("task1", 100L);
     TaskEntity task2 = createTask("task2", 101L);
     List<PolarisBaseEntity> createdEntities =
@@ -190,9 +167,6 @@ public abstract class BasePolarisMetaStoreManagerTest {
   @Test
   protected void testCreateEntitiesWithConflict() {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
-    if (CallContext.getCurrentContext() == null) {
-      CallContext.setCurrentContext(polarisTestMetaStoreManager.polarisCallContext);
-    }
     TaskEntity task1 = createTask("task1", 100L);
     TaskEntity task2 = createTask("task2", 101L);
     TaskEntity task3 = createTask("task3", 103L);
@@ -301,12 +275,10 @@ public abstract class BasePolarisMetaStoreManagerTest {
             entry ->
                 Assertions.assertThat(entry)
                     .extracting(
-                        e ->
-                            PolarisObjectMapperUtil.deserializeProperties(
-                                callCtx, e.getProperties()))
-                    .asInstanceOf(InstanceOfAssertFactories.map(String.class, String.class))
-                    .containsEntry("lastAttemptExecutorId", executorId)
-                    .containsEntry("attemptCount", "1"));
+                        PolarisBaseEntity::getPropertiesAsMap,
+                        InstanceOfAssertFactories.map(String.class, String.class))
+                    .containsEntry(PolarisTaskConstants.LAST_ATTEMPT_EXECUTOR_ID, executorId)
+                    .containsEntry(PolarisTaskConstants.ATTEMPT_COUNT, "1"));
     Set<String> firstTasks =
         taskList.stream().map(PolarisBaseEntity::getName).collect(Collectors.toSet());
 
@@ -344,7 +316,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
 
     Assertions.assertThat(emtpyList).isNotNull().isEmpty();
 
-    timeSource.add(Duration.ofMinutes(10));
+    clock.add(Duration.ofMinutes(10));
 
     // all the tasks are unassigned. Fetch them all
     List<PolarisBaseEntity> allTasks =
@@ -360,7 +332,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
     // drop all the tasks. Skip the clock forward and fetch. empty list expected
     allTasks.forEach(
         entity -> metaStoreManager.dropEntityIfExists(callCtx, null, entity, Map.of(), false));
-    timeSource.add(Duration.ofMinutes(10));
+    clock.add(Duration.ofMinutes(10));
 
     List<PolarisBaseEntity> finalList =
         metaStoreManager.loadTasks(callCtx, executorId, PageToken.fromLimit(20)).getEntities();
