@@ -20,7 +20,7 @@
 # Run without args to run all tests, or single arg for single test.
 SCRIPT_DIR=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
 
-export SPARK_VERSION=spark-3.5.5
+export SPARK_VERSION=spark-3.5.6
 export SPARK_DISTRIBUTION=${SPARK_VERSION}-bin-hadoop3
 
 if [ -z "${SPARK_HOME}" ]; then
@@ -44,6 +44,7 @@ function logred() {
 }
 
 REGTEST_HOME=$(dirname $(realpath $0))
+
 cd ${REGTEST_HOME}
 
 ./setup.sh
@@ -72,7 +73,7 @@ if [[ -z "$REGTEST_ROOT_BEARER_TOKEN" ]]; then
   if ! output=$(curl -X POST -H "Polaris-Realm: POLARIS" "http://${POLARIS_HOST:-localhost}:8181/api/catalog/v1/oauth/tokens" \
     -d "grant_type=client_credentials" \
     -d "client_id=root" \
-    -d "client_secret=secret" \
+    -d "client_secret=s3cr3t" \
     -d "scope=PRINCIPAL_ROLE:ALL"); then
     logred "Error: Failed to retrieve bearer token"
     exit 1
