@@ -18,9 +18,8 @@
  */
 
 plugins {
-  alias(libs.plugins.quarkus)
-  alias(libs.plugins.jandex)
-  id("polaris-runtime")
+  id("org.kordamp.gradle.jandex")
+  id("polaris-server")
 }
 
 dependencies {
